@@ -3,6 +3,7 @@ from typing import NotRequired, TypedDict
 from agentle.agents.whatsapp.models.audio_message import AudioMessage
 from agentle.agents.whatsapp.models.document_message import DocumentMessage
 from agentle.agents.whatsapp.models.image_message import ImageMessage
+from agentle.agents.whatsapp.models.message_context_info import MessageContextInfo
 from agentle.agents.whatsapp.models.video_message import VideoMessage
 
 
@@ -14,6 +15,8 @@ class Message(TypedDict):
         imageMessage: Dados da imagem (para mensagens com imagem)
         documentMessage: Dados do documento (para mensagens com documento)
         audioMessage: Dados do áudio (para mensagens de áudio)
+        videoMessage: Dados do vídeo (para mensagens de vídeo)
+        messageContextInfo: Informações de contexto da mensagem (opcional)
         base64: Conteúdo da mídia codificado em base64 (opcional)
     """
 
@@ -21,5 +24,6 @@ class Message(TypedDict):
     imageMessage: NotRequired[ImageMessage]
     documentMessage: NotRequired[DocumentMessage]
     audioMessage: NotRequired[AudioMessage]
-    base64: NotRequired[str]
     videoMessage: NotRequired[VideoMessage]
+    messageContextInfo: NotRequired[MessageContextInfo]
+    base64: NotRequired[str]
