@@ -23,16 +23,20 @@ from __future__ import annotations
 
 import asyncio
 import logging
-from collections.abc import Mapping, MutableSequence, Sequence
-from typing import TYPE_CHECKING, cast, override
+from collections.abc import AsyncGenerator, AsyncIterator, Mapping, Sequence
+from textwrap import dedent
+from typing import TYPE_CHECKING, cast, overload, override
 
 from agentle.generations.models.generation.generation import Generation
 from agentle.generations.models.generation.generation_config import GenerationConfig
 from agentle.generations.models.generation.generation_config_dict import (
     GenerationConfigDict,
 )
+from agentle.generations.models.message_parts.file import FilePart
+from agentle.generations.models.message_parts.text import TextPart
 from agentle.generations.models.messages.developer_message import DeveloperMessage
 from agentle.generations.models.messages.message import Message
+from agentle.generations.models.messages.user_message import UserMessage
 from agentle.generations.providers.base.generation_provider import (
     GenerationProvider,
 )
@@ -53,17 +57,17 @@ from agentle.generations.providers.google.function_calling_config import (
 )
 from agentle.generations.providers.types.model_kind import ModelKind
 from agentle.generations.tools.tool import Tool
-from agentle.generations.tracing.contracts.stateful_observability_client import (
-    StatefulObservabilityClient,
-)
-from agentle.generations.tracing.decorators.observe import observe
+from agentle.generations.tracing import observe
+from agentle.utils.describe_model_for_llm import describe_model_for_llm
 
 if TYPE_CHECKING:
     from google.auth.credentials import Credentials
     from google.genai.client import (
         DebugConfig,
     )
-    from google.genai.types import Content, GenerateContentResponse, HttpOptions
+    from google.genai.types import HttpOptions
+
+    from agentle.generations.tracing.otel_client import OtelClient
 
 
 type WithoutStructuredOutput = None
@@ -99,7 +103,7 @@ class GoogleGenerationProvider(GenerationProvider):
     def __init__(
         self,
         *,
-        tracing_client: StatefulObservabilityClient | None = None,
+        otel_clients: Sequence[OtelClient] | OtelClient | None = None,
         use_vertex_ai: bool = False,
         api_key: str | None | None = None,
         credentials: Credentials | None = None,
@@ -113,7 +117,7 @@ class GoogleGenerationProvider(GenerationProvider):
         Initialize the Google Generation Provider.
 
         Args:
-            tracing_client: Optional client for observability and tracing.
+            otel_clients: Optional client for observability and tracing.
             use_vertex_ai: Whether to use Google Vertex AI instead of standard API.
             api_key: Optional API key for authentication with Google AI.
             credentials: Optional credentials object for authentication.
@@ -127,14 +131,14 @@ class GoogleGenerationProvider(GenerationProvider):
         from google import genai
         from google.genai import types
 
-        super().__init__(tracing_client=tracing_client)
+        super().__init__(otel_clients=otel_clients)
         self.message_adapter = MessageToGoogleContentAdapter()
         self.function_calling_config = function_calling_config or {}
 
         _http_options = http_options or types.HttpOptions()
         self._client = genai.Client(
             vertexai=use_vertex_ai,
-            api_key=api_key,
+            api_key=api_key if not use_vertex_ai else None,
             credentials=credentials,
             project=project if use_vertex_ai else None,
             location=location if use_vertex_ai else None,
@@ -148,7 +152,7 @@ class GoogleGenerationProvider(GenerationProvider):
         """
         The default model to use for generation.
         """
-        return "gemini-2.0-flash"
+        return "gemini-2.5-flash"
 
     @property
     @override
@@ -161,6 +165,186 @@ class GoogleGenerationProvider(GenerationProvider):
         """
         return "google"
 
+    @overload
+    def stream_async[T](
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        response_schema: type[T],
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+    ) -> AsyncGenerator[Generation[T], None]: ...
+
+    @overload
+    def stream_async(
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+        tools: Sequence[Tool],
+    ) -> AsyncGenerator[Generation[WithoutStructuredOutput], None]: ...
+
+    @overload
+    def stream_async(
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+    ) -> AsyncGenerator[Generation[WithoutStructuredOutput], None]: ...
+
+    # Remove the explicit return type annotation from the implementation
+    async def stream_async[T = WithoutStructuredOutput](
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        response_schema: type[T] | None = None,
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+        tools: Sequence[Tool] | None = None,
+    ):
+        from google.genai import types
+
+        if self._normalize_generation_config(generation_config).n > 1:
+            raise ValueError("streaming does not support n > 1.")
+
+        used_model = self._resolve_model(model)
+        _generation_config = self._normalize_generation_config(generation_config)
+
+        system_instruction: str | None = None
+        first_message = messages[0]
+        if isinstance(first_message, DeveloperMessage):
+            system_instruction = first_message.text
+
+        if response_schema:
+            model_description = describe_model_for_llm(response_schema)  # type: ignore[reportArgumentType]
+            json_instruction = "Your Output must be a valid JSON string. Do not include any other text. You must provide an asnwer following the following json structure:"
+            conditional_prefix = (
+                "If, and only if, not calling any tools, " if tools else ""
+            )
+
+            instruction_text = (
+                f"{conditional_prefix}{json_instruction}\n{model_description}"
+            )
+
+            system_instruction = (
+                dedent(f"""\
+                You are a helpful assistant. {instruction_text}
+                """)
+                if not system_instruction
+                else system_instruction
+                + dedent(f"""\
+                \n\n
+                {instruction_text}
+                """)
+            )
+
+        message_tools = [
+            part
+            for message in messages
+            for part in message.parts
+            if isinstance(part, Tool)
+        ]
+
+        final_tools = (
+            list(tools or []) + message_tools if tools or message_tools else None
+        )
+
+        disable_function_calling = self.function_calling_config.get("disable", True)
+        # if disable_function_calling is True, set maximum_remote_calls to None
+        maximum_remote_calls = None if disable_function_calling else 10
+        ignore_call_history = self.function_calling_config.get(
+            "ignore_call_history", False
+        )
+
+        _tools: types.ToolListUnion | None = (
+            [AgentleToolToGoogleToolAdapter().adapt(tool) for tool in final_tools]
+            if final_tools
+            else None
+        )
+
+        config = types.GenerateContentConfig(
+            system_instruction=system_instruction,
+            temperature=_generation_config.temperature,
+            top_p=_generation_config.top_p,
+            top_k=_generation_config.top_k,
+            candidate_count=_generation_config.n,
+            tools=_tools,
+            max_output_tokens=_generation_config.max_output_tokens,
+            response_schema=response_schema if bool(response_schema) else None,
+            automatic_function_calling=types.AutomaticFunctionCallingConfig(
+                disable=disable_function_calling,
+                maximum_remote_calls=maximum_remote_calls,
+                ignore_call_history=ignore_call_history,
+            ),
+        )
+
+        user_messages = [msg for msg in messages if isinstance(msg, UserMessage)]
+        if user_messages and all(
+            isinstance(part, FilePart) for part in user_messages[-1].parts
+        ):
+            messages = list(messages) + [UserMessage(parts=[TextPart(text=".")])]
+
+        # Developer message is usuarlly the first message. if it's not, then there is no developer message
+        if isinstance(messages[0], DeveloperMessage):
+            messages = messages[1:]
+
+        contents = [self.message_adapter.adapt(msg) for msg in messages]
+
+        try:
+            async with asyncio.timeout(_generation_config.timeout_in_seconds):
+                generate_content_response_stream: AsyncIterator[
+                    types.GenerateContentResponse
+                ] = await self._client.aio.models.generate_content_stream(
+                    model=used_model,
+                    contents=cast(types.ContentListUnion, contents),
+                    config=config,
+                )
+        except asyncio.TimeoutError as e:
+            e.add_note(
+                f"Content generation timed out after {_generation_config.timeout_in_seconds}s"
+            )
+            raise
+
+        # Create the response
+        response = GenerateGenerateContentResponseToGenerationAdapter[T](
+            response_schema=response_schema, model=used_model
+        ).adapt(generate_content_response_stream)
+
+        # Yield from the async iterator to make this an async generator
+        async for generation in response:
+            yield generation
+
+    @overload
+    async def generate_async[T](
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        response_schema: type[T],
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+    ) -> Generation[T]: ...
+
+    @overload
+    async def generate_async(
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+        tools: Sequence[Tool],
+    ) -> Generation[WithoutStructuredOutput]: ...
+
+    @overload
+    async def generate_async(
+        self,
+        *,
+        model: str | ModelKind | None = None,
+        messages: Sequence[Message],
+        generation_config: GenerationConfig | GenerationConfigDict | None = None,
+    ) -> Generation[WithoutStructuredOutput]: ...
+
     @observe
     @override
     @override_model_kind
@@ -197,10 +381,10 @@ class GoogleGenerationProvider(GenerationProvider):
         used_model = self._resolve_model(model)
         _generation_config = self._normalize_generation_config(generation_config)
 
-        system_instruction: Content | None = None
+        system_instruction: str | None = None
         first_message = messages[0]
         if isinstance(first_message, DeveloperMessage):
-            system_instruction = self.message_adapter.adapt(first_message)
+            system_instruction = first_message.text
 
         message_tools = [
             part
@@ -243,9 +427,17 @@ class GoogleGenerationProvider(GenerationProvider):
             ),
         )
 
-        contents: MutableSequence[Content] = [
-            self.message_adapter.adapt(message) for message in messages
-        ]
+        user_messages = [msg for msg in messages if isinstance(msg, UserMessage)]
+        if user_messages and all(
+            isinstance(part, FilePart) for part in user_messages[-1].parts
+        ):
+            messages = list(messages) + [UserMessage(parts=[TextPart(text=".")])]
+
+        # Developer message is usuarlly the first message. if it's not, then there is no developer message
+        if isinstance(messages[0], DeveloperMessage):
+            messages = messages[1:]
+
+        contents = [self.message_adapter.adapt(msg) for msg in messages]
 
         try:
             async with asyncio.timeout(_generation_config.timeout_in_seconds):
@@ -270,8 +462,6 @@ class GoogleGenerationProvider(GenerationProvider):
 
         return response
 
-    def _create_med_lm_generation(self) -> GenerateContentResponse: ...
-
     @override
     def map_model_kind_to_provider_model(
         self,
@@ -279,17 +469,17 @@ class GoogleGenerationProvider(GenerationProvider):
     ) -> str:
         mapping: Mapping[ModelKind, str] = {
             # Stable models
-            "category_nano": "gemini-2.0-flash-lite",
-            "category_mini": "gemini-2.0-flash",
-            "category_standard": "gemini-2.0-flash",
+            "category_nano": "gemini-2.5-flash-lite",
+            "category_mini": "gemini-2.5-flash",
+            "category_standard": "gemini-2.5-flash",
             "category_pro": "gemini-2.5-pro",
             "category_flagship": "gemini-2.5-pro",
             "category_reasoning": "gemini-2.5-pro",
             "category_vision": "gemini-2.5-pro-vision",
             "category_coding": "gemini-2.5-pro",
-            "category_instruct": "gemini-2.0-flash",
+            "category_instruct": "gemini-2.5-flash",
             # Experimental models
-            "category_nano_experimental": "gemini-2.0-flash-lite",  # no distinct experimental found
+            "category_nano_experimental": "gemini-2.5-flash-lite",  # no distinct experimental found
             "category_mini_experimental": "gemini-2.5-flash-preview-05-20",  # real preview model
             "category_standard_experimental": "gemini-2.5-flash-preview-05-20",  # fallback
             "category_pro_experimental": "gemini-2.5-pro",  # fallback
@@ -326,8 +516,7 @@ class GoogleGenerationProvider(GenerationProvider):
             "gemini-2.5-flash": 0.15,
             "gemini-2.5-flash-preview-05-20": 0.15,  # Assuming same as standard 2.5 Flash
             # Gemini 2.0 Flash family
-            "gemini-2.0-flash": 0.15,
-            "gemini-2.0-flash-lite": 0.075,
+            "gemini-2.5-flash-lite": 0.075,
         }
 
         price_info = model_to_price_per_million.get(model)
@@ -381,8 +570,7 @@ class GoogleGenerationProvider(GenerationProvider):
             "gemini-2.5-flash": 0.60,  # Standard text output (3.50 for thinking/reasoning)
             "gemini-2.5-flash-preview-05-20": 0.60,  # Assuming same as standard 2.5 Flash
             # Gemini 2.0 Flash family
-            "gemini-2.0-flash": 0.60,
-            "gemini-2.0-flash-lite": 0.30,
+            "gemini-2.5-flash-lite": 0.30,
         }
 
         price_info = model_to_price_per_million.get(model)
