@@ -1,26 +1,21 @@
-import inspect
 from pathlib import Path
-from typing import Any, Literal, MutableMapping, cast, override
+from typing import Literal, cast
 from urllib.parse import urlparse
 
 from rsb.functions.create_instance_dynamically import create_instance_dynamically
 from rsb.models.field import Field
 
-from agentle.agents.agent import Agent
-from agentle.generations.models.structured_outputs_store.audio_description import (
-    AudioDescription,
-)
-from agentle.generations.models.structured_outputs_store.visual_media_description import (
-    VisualMediaDescription,
-)
+from agentle.generations.providers.base.generation_provider import GenerationProvider
 from agentle.parsing.document_parser import DocumentParser
-from agentle.parsing.factories.audio_description_agent_default_factory import (
-    audio_description_agent_default_factory,
-)
-from agentle.parsing.factories.visual_description_agent_default_factory import (
-    visual_description_agent_default_factory,
-)
 from agentle.parsing.parsed_file import ParsedFile
+from agentle.utils.file_validation import (
+    FileValidationError,
+    resolve_file_path,
+    validate_file_exists,
+)
+from agentle.utils.file_validation import (
+    is_url as is_valid_url,
+)
 
 
 class FileParser(DocumentParser):
@@ -76,7 +71,7 @@ class FileParser(DocumentParser):
         from agentle.generations.models.structured_outputs_store.audio_description import AudioDescription
 
         custom_audio_agent = Agent(
-            model="gemini-2.0-flash",
+            model="gemini-2.5-flash",
             instructions="Transcribe technical terminology with high accuracy",
             response_schema=AudioDescription
         )
@@ -122,7 +117,7 @@ class FileParser(DocumentParser):
     )
 
     legal_audio_agent = Agent(
-        model="gemini-2.0-flash",
+        model="gemini-2.5-flash",
         instructions="Transcribe legal terminology with high accuracy",
         response_schema=AudioDescription
     )
@@ -141,7 +136,7 @@ class FileParser(DocumentParser):
 
     type: Literal["file"] = "file"
     strategy: Literal["low", "high"] = Field(default="high")
-    visual_description_agent: Agent[VisualMediaDescription] | None = Field(
+    visual_description_provider: GenerationProvider | None = Field(
         default=None,
     )
     """
@@ -149,7 +144,7 @@ class FileParser(DocumentParser):
     Useful when you want to customize the prompt for the visual description.
     """
 
-    audio_description_agent: Agent[AudioDescription] | None = Field(
+    audio_description_provider: GenerationProvider | None = Field(
         default=None,
     )
     """
@@ -160,7 +155,6 @@ class FileParser(DocumentParser):
     parse_timeout: float = Field(default=30)
     """The timeout for the parse operation in seconds."""
 
-    @override
     async def parse_async(self, document_path: str) -> ParsedFile:
         """
         Asynchronously parse a document using the appropriate parser for its file type.
@@ -203,19 +197,40 @@ class FileParser(DocumentParser):
         from agentle.parsing.parsers.link import LinkParser
         from agentle.parsing.parses import parser_registry
 
-        path = Path(document_path)
+        # Enhanced path validation and resolution
+        try:
+            # Check if it's a URL first
+            if is_valid_url(document_path):
+                parser_cls = cast(
+                    type[DocumentParser],
+                    LinkParser,
+                )
+
+                return await create_instance_dynamically(
+                    parser_cls,
+                    visual_description_provider=self.visual_description_provider,
+                    audio_description_provider=self.audio_description_provider,
+                    parse_timeout=self.parse_timeout,
+                ).parse_async(document_path=document_path)
+
+            # For file paths, resolve and validate
+            resolved_path = resolve_file_path(document_path)
+            validate_file_exists(resolved_path)
+
+            # Use resolved path for further processing
+            path = Path(resolved_path)
+
+        except FileValidationError as e:
+            raise ValueError(
+                f"File validation failed for '{document_path}': {e}"
+            ) from e
+
         parser_cls: type[DocumentParser] | None = parser_registry.get(
             path.suffix.lstrip(".")
         )
 
-        visual_description_agent = (
-            self.visual_description_agent or visual_description_agent_default_factory()
-        )
-        audio_description_agent = (
-            self.audio_description_agent or audio_description_agent_default_factory()
-        )
-
         if not parser_cls:
+            # Double-check URL handling (fallback)
             parsed_url = urlparse(document_path)
             is_url = parsed_url.scheme in ["http", "https"]
 
@@ -225,29 +240,21 @@ class FileParser(DocumentParser):
                     LinkParser,
                 )
 
-                return await create_instance_dynamically(  # used because mypy complained about the type of the parser_cls
+                return await create_instance_dynamically(
                     parser_cls,
-                    visual_description_agent=visual_description_agent,
-                    audio_description_agent=audio_description_agent,
+                    visual_description_provider=self.visual_description_provider,
+                    audio_description_provider=self.audio_description_provider,
                     parse_timeout=self.parse_timeout,
                 ).parse_async(document_path=document_path)
             else:
-                raise ValueError(f"Unsupported extension: {path.suffix}")
+                raise ValueError(
+                    f"Unsupported file extension '{path.suffix}' for file: {resolved_path}"
+                )
 
-        # Get the signature of the parser constructor
-        parser_signature = inspect.signature(parser_cls.__init__)
-        valid_params = parser_signature.parameters.keys()
-
-        # Only include arguments that are accepted by the parser constructor
-        potential_args = {
-            "strategy": self.strategy,
-            "visual_description_agent": self.visual_description_agent,
-            "audio_description_agent": self.audio_description_agent,
-        }
-
-        kwargs: MutableMapping[str, Any] = {}
-        for arg_name, arg_value in potential_args.items():
-            if arg_name in valid_params:
-                kwargs[arg_name] = arg_value
-
-        return await parser_cls(**kwargs).parse_async(document_path)
+        return await create_instance_dynamically(
+            parser_cls,
+            visual_description_provider=self.visual_description_provider,
+            audio_description_provider=self.audio_description_provider,
+            parse_timeout=self.parse_timeout,
+            strategy=self.strategy,
+        ).parse_async(document_path=str(resolved_path))
