@@ -7,15 +7,15 @@ This example demonstrates how to use different model providers with the Agentle
 from agentle.generations.models.message_parts.text import TextPart
 from agentle.generations.models.messages.user_message import UserMessage
 from agentle.generations.providers.base.generation_provider import GenerationProvider
-from agentle.generations.providers.google.google_genai_generation_provider import (
-    GoogleGenaiGenerationProvider,
+from agentle.generations.providers.google.google_generation_provider import (
+    GoogleGenerationProvider,
 )
 
 # Example 1: Create an agent with Google's Gemini model
-provider: GenerationProvider = GoogleGenaiGenerationProvider()
+provider: GenerationProvider = GoogleGenerationProvider()
 
 # Run the Google agent
-google_response = provider.create_generation(
+google_response = provider.generate(
     messages=[
         UserMessage(
             parts=[
